package com.dbxml.db.admin.nodes;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: RolesNode.java,v 1.6 2004/07/29 18:41:15 bradford Exp $
 */

import com.dbxml.db.admin.Admin;
import com.dbxml.db.admin.dialogs.PermissionsDialog;
import com.dbxml.db.client.CollectionClient;
import com.dbxml.db.common.security.AccessManagerClient;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.swing.Icon;
import javax.swing.ImageIcon;

/**
 * RolesNode
 */

public final class RolesNode extends AdminNode implements HasCollection, HasChildren, HasMenu {
   private static final AdminNode[] EmptyAdminNodes = new AdminNode[0];

   private static final Icon RolesIcon = new ImageIcon(CollectionNode.class.getResource("roles_mini.gif"));

   public static final int ACTION_ADD = 0;

   private static final MenuItem[] Menu = new MenuItem[]{new MenuItem(ACTION_ADD, "Add Role")};

   private CollectionClient col;
   private int type;
   private String userID;

   public RolesNode(AdminNode parentNode, CollectionClient col, String userID) {
      super(parentNode);

      this.col = col;
      this.type = RoleNode.USER;
      this.userID = userID;
   }

   public RolesNode(AdminNode parentNode, CollectionClient col, int type) {
      super(parentNode);

      this.col = col;
      this.type = type;
   }

   public String getLabel() {
      switch ( type ) {
         case RoleNode.DATABASE:
            return "Database Roles";
         case RoleNode.COLLECTION:
            return "Collection Roles";
         case RoleNode.USER:
            return "User Roles";
         default:
            return null;
      }
   }

   public String getMenuTitle() {
      return "Roles";
   }

   public Icon getIcon() {
      return RolesIcon;
   }

   public CollectionClient getCollection() {
      return col;
   }

   public AdminNode[] getChildren() {
      try {
         AccessManagerClient manager = new AccessManagerClient(col.getClient());
         switch ( type ) {
            case RoleNode.DATABASE: {
               String[] roles = manager.listRoles();
               AdminNode[] list = new AdminNode[roles.length];
               for ( int i = 0; i < roles.length; i++ )
                  list[i] = new RoleNode(this, col, roles[i], type);
               return list;
            }

            case RoleNode.COLLECTION: {
               Map acl = manager.listAccessControl(col.getCanonicalName());
               List list = new ArrayList();
               Iterator iter = acl.entrySet().iterator();
               while ( iter.hasNext() ) {
                  Map.Entry entry = (Map.Entry)iter.next();
                  String role = (String)entry.getKey();
                  Integer permissions = (Integer)entry.getValue();
                  list.add(new RoleNode(this, col, role, type, permissions.intValue()));
               }
               return (AdminNode[])list.toArray(EmptyAdminNodes);
            }

            case RoleNode.USER: {
               String[] roles = manager.listRolesForUser(userID);
               AdminNode[] list = new AdminNode[roles.length];
               for ( int i = 0; i < roles.length; i++ )
                  list[i] = new RoleNode(this, col, roles[i], RoleNode.USER, userID);
               return list;
            }

            default:
               return EmptyAdminNodes;
         }
      }
      catch ( Exception e ) {
         return EmptyAdminNodes;
      }
   }

   public MenuItem[] getMenu() {
      if ( type == RoleNode.COLLECTION )
         return Menu;
      else
         return null;
   }

   public int menuAction(int action) {
      switch ( action ) {
         case ACTION_ADD:
            Admin admin = Admin.getInstance();
            if ( type == RoleNode.COLLECTION ) {
               PermissionsDialog perms = new PermissionsDialog(admin, col);
               perms.show();
            }
            break;
      }

      return REFRESH_SELF;
   }
}


