package com.dbxml.db.client.tools.command;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: CreateCollection.java,v 1.7 2004/02/12 00:17:55 bradford Exp $
 */

import com.dbxml.db.client.CollectionClient;
import com.dbxml.db.server.dbXML;
import com.dbxml.util.dbXMLException;
import com.dbxml.xml.dom.DOMHelper;
import java.util.Properties;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import java.util.Iterator;
import java.util.Map;

/**
 * CreateCollection (MKCOL) creates a Collection under the current
 * context Collection.
 */

public final class CreateCollection extends CreateBase {
   private static final String CLASS = "class";
   private static final String DEFAULT_CLASS = dbXML.DEFAULT_FILER;
   private static final String PAGESIZE = "pagesize";
   private static final String DEFAULT_PAGESIZE = "4096";
   private static final String TYPE = "type";
   private static final String DEFAULT_TYPE = "compressed";

   protected String getType() {
      return "Collection";
   }

   protected boolean createObject(CollectionClient col, String name, Properties props) throws dbXMLException {
      try {
         String cname = props.getProperty(CLASS, DEFAULT_CLASS);
         props.remove(CLASS);
         String type = props.getProperty(TYPE, DEFAULT_TYPE);
         props.remove(TYPE);

         Document doc = DOMHelper.newDocument();
         Element root = doc.createElement("collection");
         root.setAttribute("name", name);
         root.setAttribute("type", type);
         doc.appendChild(root);

         Element filer = doc.createElement("filer");
         filer.setAttribute("class", cname);
         Iterator iter = props.entrySet().iterator();
         while ( iter.hasNext() ) {
            Map.Entry entry = (Map.Entry)iter.next();
            String key = (String)entry.getKey();
            String value = (String)entry.getValue();
            filer.setAttribute(key, value);
         }
         root.appendChild(filer);

         root.appendChild(doc.createElement("indexes"));
         root.appendChild(doc.createElement("triggers"));
         root.appendChild(doc.createElement("extensions"));

         return col.createCollection(name, doc) != null;
      }
      catch ( Exception e ) {
         throw new dbXMLException(e);
      }
   }
}


