package com.dbxml.db.client.xmldb;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: FullTextQueryService.java,v 1.2 2004/02/12 00:17:55 bradford Exp $
 */

import org.xmldb.api.base.ResourceSet;
import org.xmldb.api.base.Service;
import org.xmldb.api.base.XMLDBException;

/**
 * FullTextQueryService is a <code>Service</code> that enables the execution
 * of full text querying within the context of a <code>Collection</code>.
 */

public interface FullTextQueryService extends Service {
   // Namespace Manipulation

   /**
    * Sets a namespace mapping in the internal namespace map used to evaluate
    * queries. If <code>prefix</code> is null or empty the default namespace is
    * associated with the provided URI. A null or empty <code>uri</code> results
    * in an exception being thrown.
    *
    * @param prefix The prefix to set in the map. If
    *  <code>prefix</code> is empty or null the
    *  default namespace will be associated with the provided URI.
    * @param uri The URI for the namespace to be associated with prefix.
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br />
    * @todo probably need some special error here.
    */
   void setNamespace(String prefix, String uri) throws XMLDBException;

   /**
    * Returns the URI string associated with <code>prefix</code> from
    * the internal namespace map. If <code>prefix</code> is null or empty the
    * URI for the default namespace will be returned. If a mapping for the
    * <code>prefix</code> can not be found null is returned.
    *
    * @param prefix The prefix to retrieve from the namespace map.
    * @return The URI associated with <code>prefix</code>
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br />
    */
   String getNamespace(String prefix) throws XMLDBException;

   /**
    * Removes the namespace mapping associated with <code>prefix</code> from
    * the internal namespace map. If <code>prefix</code> is null or empty the
    * mapping for the default namespace will be removed.
    *
    * @param prefix The prefix to remove from the namespace map. If
    *  <code>prefix</code> is null or empty the mapping for the default
    *  namespace will be removed.
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br />
    */
   void removeNamespace(String prefix) throws XMLDBException;

   /**
    * Removes all namespace mappings stored in the internal namespace map.
    *
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br />
    */
   void clearNamespaces() throws XMLDBException;


   // Query Invocation

   /**
    * Performs a full-text query against the <code>Collection</code>.
    * The result is a <code>ResourceSet</code> containing the results of the
    * query. Any namespaces used in the <code>query</code> string will be
    * evaluated using the mappings setup using <code>setNamespace</code>.
    *
    * @param query The query string to use.
    * @return A <code>ResourceSet</code> containing the results of the query.
    * @exception XMLDBException with expected error codes.<br />
    *  <code>ErrorCodes.VENDOR_ERROR</code> for any vendor
    *  specific errors that occur.<br />
    */
   ResourceSet query(String query) throws XMLDBException;
}
