package com.dbxml.db.client.xmlrpc;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: dbXMLClientImpl.java,v 1.4 2004/02/13 01:42:31 bradford Exp $
 */

import com.dbxml.db.client.CollectionClient;
import com.dbxml.db.client.ContentClient;
import com.dbxml.db.client.dbXMLClient;
import com.dbxml.util.dbXMLException;

/**
 * dbXMLClientImpl
 */

public final class dbXMLClientImpl extends ConnectionManager implements dbXMLClient {
   private static final String PATH = "dbXML";

   private boolean connected;
   private CollectionClient database;

   public dbXMLClientImpl() {
   }

   public void connect() throws dbXMLException {
      if ( connected )
         throw new dbXMLException("Already connected");

      try {
         setClientPath(PATH);
         getXmlRpcClient();
         getDatabase(); // to Test
         connected = true;
      }
      catch ( dbXMLException e ) {
         connected = false;
         throw e;
      }
      catch ( Exception e ) {
         connected = false;
         throw new dbXMLException(e);
      }
   }

   public void disconnect() throws dbXMLException {
      if ( connected ) {
         connected = false;
         database = null;
      }
      else
         throw new dbXMLException("Not connected");
   }

   public String getServerVersion() throws dbXMLException {
      return executeString("getServerVersion", null);
   }

   public void shutdown(int exitCode) throws dbXMLException {
      try {
         execute("shutdown", new Object[]{new Integer(exitCode)});
      }
      catch ( dbXMLException e ) {
         if ( e.getFaultCode() == 0 )
            return;
         else
            throw e;
      }
   }

   public CollectionClient getDatabase() throws dbXMLException {
      if ( database == null )
         database = new CollectionClientImpl(this, "/", false);
      return database;
   }

   public CollectionClient getCollection(String childPath) throws dbXMLException {
      return new CollectionClientImpl(this, childPath);
   }

   public ContentClient getContent(String docPath) throws dbXMLException {
      ContentClient doc = new ContentClientImpl(this, docPath);
      return doc;
   }
}

