package com.dbxml.db.common.query;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: QueryBase.java,v 1.3 2004/02/12 00:17:56 bradford Exp $
 */

import com.dbxml.db.core.Collection;
import com.dbxml.db.core.DBException;
import com.dbxml.db.core.FaultCodes;
import com.dbxml.db.core.data.Key;
import com.dbxml.db.core.indexer.IndexManager;
import com.dbxml.db.core.query.CompilationException;
import com.dbxml.db.core.query.Query;
import com.dbxml.db.core.query.QueryException;
import com.dbxml.xml.NamespaceMap;
import com.dbxml.xml.SymbolTable;

/**
 * QueryBase
 */

public abstract class QueryBase implements Query {
   protected Collection context;
   protected NamespaceMap nsMap;
   protected String query;
   protected Key[] keys;
   protected SymbolTable symbols;
   protected IndexManager idxMgr;

   public QueryBase(Collection context, String query, NamespaceMap nsMap, Key[] keys) throws QueryException {
      this.context = context;
      this.query = query;
      this.nsMap = nsMap;
      this.keys = keys;

      try {
         symbols = context.getSymbols();
         idxMgr = context.getIndexManager();
      }
      catch ( DBException e ) {
         throw new QueryException(FaultCodes.QRY_NOT_SUPPORTED, e);
      }
   }

   public String getQueryString() {
      return query;
   }

   public Collection getQueryContext() {
      return context;
   }

   public NamespaceMap getNamespaceMap() {
      return nsMap;
   }

   public Key[] getKeySet() {
      return keys;
   }

   public abstract void compileQuery() throws CompilationException;
}
