package com.dbxml.db.common.security;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: SecurityManagerBase.java,v 1.2 2004/02/12 00:17:56 bradford Exp $
 */

import com.dbxml.db.core.Database;
import com.dbxml.db.core.FaultCodes;
import com.dbxml.db.core.security.SecurityException;
import com.dbxml.db.core.security.SecurityManager;
import com.dbxml.db.core.security.User;
import com.dbxml.db.core.security.UserStack;
import com.dbxml.util.SimpleConfigurable;

/**
 * SecurityManagerBase
 */

public abstract class SecurityManagerBase extends SimpleConfigurable implements SecurityManager {
   protected Database database;
   protected UserStack userStack;
   protected User magicUser;

   public void setDatabase(Database database) {
      this.database = database;
   }

   public void setMagicUser(User magicUser) throws SecurityException {
      if ( this.magicUser == null )
         this.magicUser = magicUser;
      else
         throw new SecurityException(FaultCodes.SEC_INVALID_ACCESS, "Can't overwrite Magic User");
   }

   public void setUserStack(UserStack userStack) throws SecurityException {
      if ( this.userStack == null )
         this.userStack = userStack;
      else
         throw new SecurityException(FaultCodes.SEC_INVALID_ACCESS, "Can't overwrite User Stack");
   }

   public String getCurrentUserID() {
      User user = userStack.getCurrentUser();
      if ( user != null )
         return user.getId();
      else
         return null;
   }
}
