package com.dbxml.db.common.xslt;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: XSLTResultSet.java,v 1.3 2004/02/12 00:17:56 bradford Exp $
 */

import org.w3c.dom.Node;

import com.dbxml.db.common.query.ResultSetBase;
import com.dbxml.db.core.Collection;
import com.dbxml.db.core.data.Key;
import com.dbxml.db.core.query.ProcessingException;
import com.dbxml.db.core.query.Query;
import com.dbxml.db.core.query.QueryException;
import com.dbxml.db.core.transaction.Transaction;
import com.dbxml.xml.dtsm.DTSMException;
import com.dbxml.xml.dtsm.DTSMHelper;
import com.dbxml.xml.dtsm.DocumentTable;

/**
 * XSLTResultSet
 */

final class XSLTResultSet extends ResultSetBase {
   private static final byte[] EmptyBytes = new byte[0];
   private static final Key EmptyKey = new Key(EmptyBytes);
   private static final Key[] EmptyKeys = new Key[] {EmptyKey};

   private Node res;
   private DocumentTable dt;

   public XSLTResultSet(Transaction tx, Collection context, Query query, Node res) throws QueryException {
      super(tx, context, query, EmptyKeys);
      this.res = res;
   }

   public boolean next() throws QueryException {
      checkOpened();

      if ( nextKey() != null ) {
         try {
            dt = DTSMHelper.nodeToTable(res, symbols);
            return true;
         }
         catch ( DTSMException e ) {
            throw new ProcessingException(e);
         }
      }
      else {
         dt = null;
         return false;
      }
   }

   public int getResultType() throws QueryException {
      checkOpened();
      if ( dt != null )
         return dt.getEntry(0).getTypeID();
      else
         return -1;
   }

   public DocumentTable getResult() throws QueryException {
      checkOpened();
      return dt;
   }
}
