package com.dbxml.db.core;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: Cache.java,v 1.2 2004/02/12 00:17:56 bradford Exp $
 */

import java.util.Map;

import com.dbxml.db.core.data.Key;
import com.dbxml.db.core.data.Record;
import com.dbxml.util.SoftHashMap;
import com.dbxml.xml.dtsm.DocumentTable;

/**
 * Cache implements a basic cache for Collections.
 */

public final class Cache {
   private Map table = new SoftHashMap();

   public Object get(Collection col, Key key) {
      return table.get(new CacheKey(col, key));
   }

   public Record getRecord(Collection col, Key key) {
      return (Record)get(col, key);
   }

   public DocumentTable getDocument(Collection col, Key key) {
      Object v = get(col, key);
      if ( v instanceof DocumentTable )
         return (DocumentTable)v;
      else
         return null;
   }

   public void put(Collection col, Key key, Object obj) {
      table.put(new CacheKey(col, key), obj);
   }

   public void put(Collection col, Record record) {
      put(col, record.getKey(), record);
   }

   public void remove(Collection col, Key key) {
      table.remove(new CacheKey(col, key));
   }


   /**
    * CacheKey
    */

   private class CacheKey {
      private Collection col;
      private String strVal;
      private Key key;

      public CacheKey(Collection col, Key key) {
         this.col = col;
         this.key = key;
      }

      public Collection getCollection() {
         return col;
      }

      public Key getKey() {
         return key;
      }

      public String toString() {
         if ( strVal == null )
            strVal = col.getCanonicalDocumentName(key);
         return strVal;
      }

      public int hashCode() {
         if ( strVal == null )
            strVal = col.getCanonicalDocumentName(key);
         return strVal.hashCode();
      }

      public boolean equals(Object o) {
         CacheKey comp = (CacheKey)o;
         if ( col != comp.col )
            return false;
         return key.equals(comp.key);
      }
   }
}
