package com.dbxml.db.core.data;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: Record.java,v 1.2 2004/02/12 00:17:57 bradford Exp $
 */


/**
 * Record is a container for Key/Value pairs.  Record also provides
 * metadata for a Value stored in the database.
 */

public final class Record {
   private RecordMetaData meta;
   private Value value;
   private Key key;

   public Record() {
   }

   public Record(Key key, Value value, RecordMetaData meta) {
      this.key = key;
      this.value = value;
      this.meta = meta;
   }

   public Record(Key key, Value value) {
      this.key = key;
      this.value = value;
   }

   /**
    * getKey returns the Record's Key.
    *
    * @return The Record's Key
    */
   public Key getKey() {
      return key;
   }

   /**
    * setValue sets the Record's Value.
    *
    * @param value The new Value
    */
   public void setValue(Value value) {
      this.value = value;
   }

   /**
    * setValue sets the Record's Value.
    *
    * @param value The new Value
    */
   public void setValue(String value) {
      this.value = new Value(value);
   }

   /**
    * getValue returns the Record's Value.
    *
    * @return The Record's Value
    */
   public Value getValue() {
      return value;
   }

   /**
    * getMetaData returns metadata about the Record.
    *
    * @return The metadata instance (or null)
    */
   public RecordMetaData getMetaData() {
      return meta;
   }
}
