package com.dbxml.db.core.data;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: RecordMetaData.java,v 1.2 2004/02/12 00:17:57 bradford Exp $
 */

import java.util.Map;
import java.util.TreeMap;

/**
 * RecordMetaData is a container for Record meta data, such as creation
 * and modification times.
 */

public class RecordMetaData {
   private static final String[] EmptyStrings = new String[0];

   public static final String CREATED = "created";
   public static final String MODIFIED = "modified";
   public static final String CREATOR = "creator";
   public static final String OWNER = "owner";
   public static final String GROUP = "group";
   public static final String SIZE = "size";
   public static final String NOTES = "notes";

   protected Map meta;

   public RecordMetaData() {
      meta = new TreeMap();
   }

   public RecordMetaData(Map meta) {
      this.meta = meta;
   }

   /**
    * getNames returns the names for all meta Values in the set
    *
    * @return The Value names
    */
   public final String[] getNames() {
      return (String[])meta.keySet().toArray(EmptyStrings);
   }

   /**
    * getValue returns a meta Value by name.
    *
    * @param name The value name
    * @return The metadata value
    */
   public final Object getValue(String name) {
      return meta.get(name);
   }
}
