package com.dbxml.db.core.query;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: Query.java,v 1.2 2004/02/12 00:17:57 bradford Exp $
 */

import com.dbxml.db.core.Collection;
import com.dbxml.db.core.data.Key;
import com.dbxml.db.core.transaction.Transaction;
import com.dbxml.xml.NamespaceMap;

/**
 * Query represents a compiled (parsed and primed) query.
 */

public interface Query {
   static final String NSURI = "http://www.dbxml.com/db/query";
   static final String PREFIX = "query";

   /**
    * getQueryStyle returns the QueryStyle (ex: XPath) for this Query
    * instance.
    *
    * @return The Query style
    */
   String getQueryStyle();

   /**
    * getQueryContext returns the Collection context for this query.
    *
    * @return The Collection context
    */
   Collection getQueryContext();

   /**
    * getQueryString returns the String that was used to create the
    * Query instance.
    *
    * @return The Query String
    */
   String getQueryString();

   /**
    * getNamespaceMap returns the Map for prefixed namespace
    * resolution in relation to this Query.
    *
    * @return The namespace Map (if any)
    */
   NamespaceMap getNamespaceMap();

   /**
    * getKeySet returns the initial set of Keys for this Query.
    *
    * @return The initial Key set (if any)
    */
   Key[] getKeySet();

   /**
    * execute actually Executes the query and returns the resulting
    * ResultSet.
    *
    * @param tx The controlling Transaction
    * @return The ResultSet
    */
   ResultSet execute(Transaction tx) throws QueryException;
}
