package com.dbxml.db.core.query.plan;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: Container.java,v 1.2 2004/02/12 00:17:57 bradford Exp $
 */

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/**
 * Container
 */

public class Container extends QueryNode {
   private static final QueryNode[] EmptyQueryNodes = new QueryNode[0];

   private List values = new ArrayList();

   public Container() {
   }

   public Container(QueryNode[] values) {
      for ( int i = 0; i < values.length; i++ )
         this.values.add(values[i]);
   }

   public void addQueryNode(QueryNode value) {
      values.add(value);
   }

   public boolean contains(QueryNode value) {
      return values.contains(value);
   }

   public QueryNode getQueryNode(int index) {
      return (QueryNode)values.get(index);
   }

   public QueryNode[] getQueryNodes() {
      return (QueryNode[])values.toArray(EmptyQueryNodes);
   }

   public Iterator iterator() {
      return Collections.unmodifiableList(values).iterator();
   }

   public boolean equals(Object obj) {
      if ( obj instanceof Container ) {
         Container c = (Container)obj;
         if ( values.size() != c.values.size() )
            return false;

         for ( int i = 0; i < values.size(); i++ )
            if ( !values.get(i).equals(c.values.get(i)) )
                  return false;

         return true;
      }
      else
         return false;
   }
}
