package com.dbxml.db.core.transaction;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: TransactionLog.java,v 1.2 2004/02/12 00:17:57 bradford Exp $
 */

import java.util.Iterator;

/**
 * TransactionLog represents a dbXML Transaction Log
 */

public interface TransactionLog {
   /**
    * start Starts a transaction participation for the current
    * Collection and binds the specified Transaction to the
    * current Thread.  More than one Transaction may be bound
    * to the current Thread ay any one time.
    *
    * @param tx The Transaction to bind
    */
   void start(Transaction tx) throws TransactionException;

   /**
    * commit ends a transaction participation for the current
    * Collection and Thread.  Any log entries that have accumulated
    * will be commited to the Collection.
    *
    * @param tx The Transaction to commit
    */
   void commit(Transaction tx) throws TransactionException;

   /**
    * cancel ends a transaction participation for the current
    * Collection and Thread.  Any log entries that have accumulated
    * will be rolled back (thrown away).
    *
    * @param tx The Transaction to cancel
    */
   void cancel(Transaction tx) throws TransactionException;

   /**
    * getTransactionCount returns the number of active Transactions
    * that have registered with this Transaction Log.
    *
    * @return The Transaction count
    */
   int getTransactionCount() throws TransactionException;

   /**
    * getTransactions returns an Iterator of the active Transactions
    * that have registered with this Transaction Log.
    *
    * @return An Iterator over the active Transactions
    */
   Iterator getTransactions() throws TransactionException;
}
