package com.dbxml.db.enterprise.sync;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: SyncTrigger.java,v 1.2 2004/02/12 00:17:57 bradford Exp $
 */

import java.util.Date;

import com.dbxml.db.core.Collection;
import com.dbxml.db.core.DBException;
import com.dbxml.db.core.SystemCollection;
import com.dbxml.db.core.data.Key;
import com.dbxml.db.core.transaction.Transaction;
import com.dbxml.db.core.trigger.SimpleTrigger;

/**
 * SyncTrigger
 */

public final class SyncTrigger extends SimpleTrigger {
   private static final String SYNCMGR_OBJ = "syncmanager";

   private SyncManager mgr;

   public void setCollection(Collection collection) {
      super.setCollection(collection);
      try {
         SystemCollection sysCol = getCollection().getSystemCollection();
         mgr = (SyncManager)sysCol.getExtensionManager().get(SYNCMGR_OBJ);
      }
      catch ( DBException e ) {
         System.err.println("ERROR: Can't retrieve SyncManager Extension");
         e.printStackTrace(System.err);
      }
   }

   public void afterDelete(Transaction tx, Key key, Object oldObj) throws DBException {
      Content c = mgr.getContent(collection.getCanonicalDocumentName(key), true);
      c.setStatus(Constants.STATUS_DELETED);
      mgr.storeContent(c);
   }

   public void afterUpdate(Transaction tx, Key key, Object oldObj, Object newObj) throws DBException {
      Content c = mgr.getContent(collection.getCanonicalDocumentName(key), true);
      c.setModified(new Date());
      mgr.storeContent(c);
   }

   public void afterInsert(Transaction tx, Key key, Object newObj) throws DBException {
      Content c = new Content(collection.getCanonicalDocumentName(key));
      mgr.storeContent(c);
   }
}
