package com.dbxml.db.server;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: Task.java,v 1.2 2004/02/12 00:17:57 bradford Exp $
 */

/**
 * Task is an interface used to implement light-weight tasks for the
 * dbXML interval-based scheduler. Tasks are run synchronously
 * within the scheduler, so runTask should exit as soon as possible in
 * order to return control to the scheduler. Heavy-weight tasks should be
 * implemented as Runnable and be spawned as a new Thread from the
 * light-weight task.
 *
 * @see com.dbxml.db.server.Server#addTask
 * @see com.dbxml.db.server.Server#removeTask
 */

public interface Task {
   /**
    * runTask is the entry point for a dbXML interval-based Task.
    * This method should return quickly, either by processing only parts of a
    * larger task or by spawning a worker thread.
    */
   void runTask();
}
