package com.dbxml.db.server.labrador;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: ServerProxy.java,v 1.3 2004/02/12 22:37:54 bradford Exp $
 */

import com.dbxml.db.core.Collection;
import com.dbxml.db.core.DBException;
import com.dbxml.db.core.Database;
import com.dbxml.db.core.security.Access;
import com.dbxml.db.server.Server;
import com.dbxml.db.server.dbXML;

/**
 * ServerProxy provides server management capabilities.  At the moment,
 * this only includes shutting the server down.
 */

public final class ServerProxy {
   private Server server;

   public ServerProxy(Server server) {
      this.server = server;
   }

   public String getServerVersion() {
      return dbXML.VersionString;
   }
   
   public static final String[] PARAMS_shutdown = {"exitCode"};

   public void shutdown(int exitCode) throws DBException {
		// Do an access control check
		Database db = Database.getInstance();
		if ( db != null ) {
			Collection col = db.getSystemCollection();
			String path = col.getCanonicalName();
			db.getSecurityManager().access(path, Access.EXECUTE);
		}
		server.shutDown(exitCode);
   }
}
