package com.dbxml.util;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: Configurable.java,v 1.2 2004/02/12 00:17:58 bradford Exp $
 */


/**
 * Configurable is a simple interface used to pass configuration information
 * to an object.  The implementing object is ultimately responsible for
 * maintaining any of that configuration information in any fashion it so
 * desires.  Most will create a Configuration instance to manage the data
 * in a consistent, and easy to access fashion.
 * <br><br>
 * A Configurable object should be considered 'configured' after receiving
 * a call to setConfig().  No additional configuration activities should be
 * expected of the instance, and so setConfig() should be the last call in
 * any set of configuration commands against an object.
 *
 * @see com.dbxml.util.Configuration
 */

public interface Configurable {
   /**
    * setConfig sets the configuration information for the Configurable
    * object instance.
    *
    * @param config The configuration Node
    */
   void setConfig(Configuration config) throws dbXMLException;

   /**
    * getConfig retrieves the configuration information for the
    * Configurable object instance.
    *
    * @return The configuration Node
    */
   Configuration getConfig();
}
