package com.dbxml.util;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: StringUtilities.java,v 1.2 2004/02/12 00:17:58 bradford Exp $
 */



/**
 * StringUtilities provides a set of commonly used String
 * processing routines.
 */

public final class StringUtilities {
   private StringUtilities() {
   }

   /**
    * leftJustify left-justifies a String value, space padding to width
    * characters if the string is less than width.
    *
    * @param value The value to left-justify
    * @param width The width to left-justify to
    * @return The left-justified value
    */
   public static String leftJustify(String value, int width) {
      StringBuffer sb = new StringBuffer(width);
      sb.append(value);
      for ( int i = sb.length(); i < width; i++ )
         sb.append(' ');
      if ( sb.length() > width )
         sb.setLength(width);
      return sb.toString();
   }

   /**
    * rightJustify right-justifies a String value, space padding to
    * width characters if the string is less than width.
    *
    * @param value The value to right-justify
    * @param width The width to right-justify to
    * @return The right-justified value
    */
   public static String rightJustify(String value, int width) {
      StringBuffer sb = new StringBuffer(width);
      sb.append(value);
      for ( int i = sb.length(); i < width; i++ )
         sb.insert(0, ' ');
      if ( sb.length() > width )
         return sb.toString().substring(sb.length() - width);
      else
         return sb.toString();
   }
}
