package com.dbxml.xml.dtsm;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: TableCompressor.java,v 1.3 2004/02/12 00:17:58 bradford Exp $
 */

import java.io.ByteArrayOutputStream;
import java.io.IOException;

/**
 * TableCompressor
 */

public final class TableCompressor {
   private ByteArrayOutputStream bos;
   private DocumentTable table;

   public TableCompressor(DocumentTable table, int dataSize) {
      this.table = table;
      int size = (table.getEntryCount() * Constants.DOC_ENTRY_SIZE)
         + (table.getValueCount() * Constants.VAL_ENTRY_SIZE)
         + dataSize + 8;
      bos = new ByteArrayOutputStream(size);
   }

   public TableCompressor(DocumentTable table) {
      this(table, table.getValueCount() * 64);
   }

   public TableCompressor(TableBuilder builder) {
      this(builder.buildDocumentTable(), builder.getDataSize());
   }

   public CompressedTable buildCompressedTable() throws DTSMException {
      try {
         int entryCount = table.getEntryCount();
         int valCount = table.getValueCount();

         bos.write((entryCount >>> 24) & 0xFF);
         bos.write((entryCount >>> 16) & 0xFF);
         bos.write((entryCount >>> 8) & 0xFF);
         bos.write((entryCount >>> 0) & 0xFF);

         bos.write((valCount >>> 24) & 0xFF);
         bos.write((valCount >>> 16) & 0xFF);
         bos.write((valCount >>> 8) & 0xFF);
         bos.write((valCount >>> 0) & 0xFF);

         for ( int i = 0; i < entryCount; i++ )
            table.getEntry(i).writeTo(bos);

         for ( int i = 0; i < valCount; i++ )
            table.getValueEntry(i).writeTo(bos);

         for ( int i = 0; i < valCount; i++ )
            table.getValue(i).streamTo(bos);

         return new CompressedTable(bos.toByteArray(), table.getSymbolTable());
      }
      catch ( IOException e ) {
         // This actually can't happen, but whatever
         throw new DTSMException(e);
      }
   }
}
