package com.dbxml.xml.dtsm;

/*
 * dbXML - Native XML Database
 * Copyright (C) 1999-2004  The dbXML Group, L.L.C.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: TableGenerator.java,v 1.2 2004/02/12 00:17:58 bradford Exp $
 */

import java.util.HashMap;
import java.util.Map;

import com.dbxml.xml.SymbolTable;

/**
 * TableGenerator
 */

public abstract class TableGenerator {
   protected Map props = new HashMap();
   protected SymbolTable symbols;

   /**
    * setProperty sets a named property value for this
    * TableGenerator.
    *
    * @param name the property name
    * @param value the property value
    */
   public void setProperty(String name, Object value) {
      props.put(name, value);
   }

   /**
    * getProperty returns a named property value for this
    * TableGenerator.
    *
    * @param name the property name
    * @return the property value
    */
   public Object getProperty(String name) {
      return props.get(name);
   }

   /**
    * getSymbolTable returns the SymbolTable that is associated
    * with this TableGenerator.
    *
    * @return the SymbolTable
    */
   public SymbolTable getSymbolTable() {
      return symbols;
   }

   /**
    * setSymbolTable sets the SymbolTable that will be associated
    * with this TableGenerator.
    *
    * @param symbols the SymbolTable
    */
   public void setSymbolTable(SymbolTable symbols) {
      this.symbols = symbols;
   }

   /**
    * process effectively kicks off the generation process and when
    * complete, returns the new DocumentTable instance.
    *
    * @return The DocumentTable
    */
   public abstract DocumentTable process() throws DTSMException;
}
